@@ -1,3 +1,6 @@
+from datetime import datetime, date
+
+
 class ModelField(object):
 
     class NotNullableFieldException(Exception):
@@ -79,7 +82,7 @@ class CharField(ModelField):
         self.max_length = max_length
 
         # If null is allowed, default value is None
-        if self.null:
+        if self.null and not self.default:
             self.text = None
         else:
             # If default value was set
@@ -143,7 +146,7 @@ class NumberField(ModelField):
 
 
         # If null is allowed, default value is None
-        if self.null:
+        if self.null and not self.default:
             self.number = None
         else:
             # If default value was set
@@ -184,7 +187,7 @@ class NumberField(ModelField):
             if isinstance(number, int) or isinstance(number, float):
                 self.number = args[0]
             else:
-                raise NumberField.WrongInputTypeException()
+                raise NumberField.WrongInputTypeException
 
     def __eq__(self, other):
         """
@@ -196,22 +199,126 @@ class NumberField(ModelField):
             return False
 
 
+class DatetimeField(ModelField):
+
+    DATE_FORMAT = '%Y-%m-%d %H:%M:%S'
+
+    def __init__(self, **kwargs):
+        """
+        """
+
+        super(DatetimeField, self).__init__(**kwargs)
+
+        if self.null and not self.default:
+            self.time = None
+        else:
+            if self.default:
+                self.time = self.default
+            else:
+                self.time = datetime.now()
+
+    def dumps(self):
+        """
+        """
+
+        return u'%s' % self.time.strftime(DatetimeField.DATE_FORMAT)
+
+    def loads(self, date_string):
+        """
+        """
+
+        self.time = datetime.strptime(date_string, DatetimeField.DATE_FORMAT)
+
+    def validate(self):
+        """
+        """
+
+    def set(self, *args, **kwargs):
+        """
+        """
+
+        if len(args) is 1:
+            time = args[0]
+            if isinstance(args, basestring):
+                self.loads(time)
+            else:
+                self.time = time
+
+    def get(self):
+        """
+        """
+
+        return self.time
+
+    def __eq__(self, other):
+        """
+        """
+
+        if super(DatetimeField, self).__eq__(other):
+            return self.time == other.time
+        else:
+            return False
+
+
 class DateField(ModelField):
 
+    DATE_FORMAT = '%Y-%m-%d'
+
     def __init__(self, **kwargs):
         """
         """
 
         super(DateField, self).__init__(**kwargs)
 
-        if self.null:
+        if self.null and not self.default:
             self.date = None
         else:
             if self.default:
                 self.date = self.default
             else:
-                # self.date = Date
-                pass
+                self.date = date.today()
+
+    def dumps(self):
+        """
+        """
+
+        return u'%s' % self.date.strftime(DateField.DATE_FORMAT)
+
+    def loads(self, date_string):
+        """
+        """
+
+        self.date = datetime.strptime(date_string, DateField.DATE_FORMAT)
+
+    def validate(self):
+        """
+        """
+
+    def set(self, *args, **kwargs):
+        """
+        """
+
+        if len(args) is 1:
+            date = args[0]
+            if isinstance(args, basestring):
+                self.loads(date)
+            else:
+                self.date = date
+
+    def get(self):
+        """
+        """
+
+        return self.date
+
+    def __eq__(self, other):
+        """
+        """
+
+        if super(DateField, self).__eq__(other):
+            return self.date == other.date
+        else:
+            return False
 
 
 class ForeignKeyField(ModelField):
@@ -224,7 +331,7 @@ class ForeignKeyField(ModelField):
 
 
         # If null is allowed, default value is None
-        if self.null:
+        if self.null and not self.default:
             self.relation_model = None
         else:
             # If default value was set
@@ -253,7 +360,7 @@ class ForeignKeyField(ModelField):
         """
 
         if self.relation_model is None and self.null is False:
-            raise NumberField.NotNullableFieldException()
+            raise ForeignKeyField.NotNullableFieldException()
 
         if self.relation_model:
             pass
