@@ -1,7 +1,8 @@
 import unittest
+import datetime
 
 from arangodb.api import Client, Database, Collection, Document
-from arangodb.orm.fields import CharField, ForeignKeyField
+from arangodb.orm.fields import CharField, ForeignKeyField, NumberField, DatetimeField, DateField
 from arangodb.orm.models import CollectionModel
 from arangodb.query.advanced import Query, Traveser
 from arangodb.query.utils.document import create_document_from_result_dict
@@ -202,10 +203,15 @@ class AqlQueryTestCase(ExtendedTestCase):
 
         docs = q.execute()
 
-        # for doc in docs:
-        #     print(doc.get_attributes())
-        #
-        # self.assertEqual(len(docs), 2)
+        self.assertEqual(len(docs), 2)
+
+        doc1 = docs[0]
+        doc2 = docs[1]
+
+        self.assertNotEqual(doc1.id, doc2.id)
+
+        self.assertEqual(doc1.little_number, 33)
+        self.assertEqual(doc2.little_number, 33)
 
     def test_exclude_document_from_list(self):
         q = Query()
@@ -249,10 +255,12 @@ class SimpleQueryTestCase(ExtendedTestCase):
 
         self.col1_doc1 = self.test_1_col.create_document()
         self.col1_doc1.ta='fa'
+        self.col1_doc1.bla='aaa'
         self.col1_doc1.save()
 
         self.col1_doc2 = self.test_1_col.create_document()
         self.col1_doc2.ta='fa'
+        self.col1_doc2.bla='xxx'
         self.col1_doc2.save()
 
         self.col2_doc1 = self.test_2_col.create_document()
@@ -273,6 +281,13 @@ class SimpleQueryTestCase(ExtendedTestCase):
 
         self.assertDocumentsEqual(doc, self.col1_doc1)
 
+    def test_get_no_document(self):
+        doc = SimpleQuery.get_by_example(collection=self.test_1_col, example_data={
+            '_key': 'dddd',
+        })
+
+        self.assertEqual(doc, None)
+
     def test_get_all_documents(self):
 
         docs = SimpleQuery.all(collection=self.test_1_col)
@@ -284,10 +299,38 @@ class SimpleQueryTestCase(ExtendedTestCase):
 
         self.assertNotEqual(doc1, doc2)
 
+    def test_update_document(self):
+        SimpleQuery.update_by_example(collection=self.test_1_col, example_data={ 'bla': 'xxx' }, new_value={
+            'bla': 'ttt'
+        })
+
+        self.col1_doc2.retrieve()
+
+        self.assertEqual(self.col1_doc2.bla, 'ttt')
+
+    def test_replace_document(self):
+        SimpleQuery.replace_by_example(collection=self.test_1_col, example_data={ 'bla': 'xxx' }, new_value={
+            'test': 'foo'
+        })
+
+        self.col1_doc2.retrieve()
+
+        self.assertEqual(self.col1_doc2.test, 'foo')
+
+    def test_remove_document(self):
+        SimpleQuery.remove_by_example(collection=self.test_1_col, example_data={ 'bla': 'xxx' })
+
+        all_docs = self.test_1_col.documents()
+
+        self.assertEqual(len(all_docs), 1)
+
+        doc = all_docs[0]
+        self.assertDocumentsEqual(doc, self.col1_doc1)
+
 
 class TraveserTestCase(ExtendedTestCase):
     def setUp(self):
-        self.database_name = 'testcase_simple_query_123'
+        self.database_name = 'testcase_traverser_query_123'
         self.db = Database.create(name=self.database_name)
 
         # Create collections
@@ -449,9 +492,58 @@ class CollectionModelTestCase(unittest.TestCase):
         TestModel.destroy()
 
 
+class CollectionModelManagerTestCase(unittest.TestCase):
+    def setUp(self):
+        self.database_name = 'testcase_collection_model_manager_123'
+        self.db = Database.create(name=self.database_name)
+
+    def tearDown(self):
+        Database.remove(name=self.database_name)
+
+    def test_retrieve_all_models(self):
+
+        class TestModel(CollectionModel):
+            pass
+
+        TestModel.init()
+
+        model1 = TestModel()
+        model1.save()
+
+        model2 = TestModel()
+        model2.save()
+
+        all_models = TestModel.objects.all()
+
+        self.assertEqual(len(all_models), 2)
+
+        TestModel.destroy()
+
+    def test_iterate_over_queryset(self):
+
+        class TestModel(CollectionModel):
+            pass
+
+        TestModel.init()
+
+        model1 = TestModel()
+        model1.save()
+
+        model2 = TestModel()
+        model2.save()
+
+        all_models = TestModel.objects.all()
+
+        for model in all_models:
+            self.assertTrue(isinstance(model, TestModel))
+
+        TestModel.destroy()
+
+
+
 class CollectionModelForeignKeyFieldTestCase(unittest.TestCase):
     def setUp(self):
-        self.database_name = 'testcase_foreign_key_field_123'
+        self.database_name = 'testcase_collection_model_foreign_key_field_123'
         self.db = Database.create(name=self.database_name)
 
     def tearDown(self):
@@ -494,6 +586,161 @@ class CollectionModelForeignKeyFieldTestCase(unittest.TestCase):
         TestModel.destroy()
 
 
+class CharFieldTestCase(unittest.TestCase):
+
+    def setUp(self):
+        pass
+
+    def tearDown(self):
+        pass
+
+    def test_equals(self):
+
+        str1 = CharField()
+        str1.set("jjj")
+
+        str2 = CharField()
+        str2.set("jjj")
+
+        self.assertEqual(str1, str2)
+
+
+class NumberFieldTestCase(unittest.TestCase):
+
+    def setUp(self):
+        pass
+
+    def tearDown(self):
+        pass
+
+    def test_basic_creation_with_default(self):
+        number = 122
+        field = NumberField(default=number)
+
+        self.assertEqual(number, field.number)
+
+    def test_equals(self):
+
+        number1 = NumberField(null=False)
+        number1.set(23)
+
+        number2 = NumberField(null=False)
+        number2.set(23)
+
+        self.assertEqual(number1, number2)
+
+    def test_wrong_input(self):
+
+        number1 = NumberField()
+
+        try:
+            number1.set("ddd")
+            self.assertTrue(False, msg='There should have been a NumberField.NotNullableFieldException')
+        except:
+            self.assertTrue(True)
+
+
+class DateFieldTestCase(unittest.TestCase):
+
+    def setUp(self):
+        pass
+
+    def tearDown(self):
+        pass
+
+    def test_basic_creation_with_default(self):
+        date = datetime.date.today()
+        field = DateField(default=date)
+
+        self.assertEqual(date, field.date)
+
+    def test_equals(self):
+        date = datetime.date.today()
+        field1 = DateField()
+        field1.set(date)
+
+        field2 = DateField()
+        field2.set(date)
+
+        self.assertEqual(field1, field2)
+
+    def test_not_equals(self):
+
+        field1 = DateField(null=False)
+        field2 = DateField(null=False)
+
+        self.assertTrue(field1 != field2)
+
+
+class DatetimeFieldTestCase(unittest.TestCase):
+
+    def setUp(self):
+        pass
+
+    def tearDown(self):
+        pass
+
+    def test_basic_creation_with_default(self):
+        time = datetime.datetime.now()
+        field = DatetimeField(default=time)
+
+        self.assertEqual(time, field.time)
+
+    def test_equals(self):
+        time = datetime.datetime.now()
+
+        field1 = DatetimeField()
+        field1.set(time)
+
+        field2 = DatetimeField()
+        field2.set(time)
+
+        self.assertEqual(field1, field2)
+
+    def test_not_equals(self):
+
+        field1 = DatetimeField(null=False)
+        field2 = DatetimeField(null=False)
+
+        self.assertTrue(field1 != field2)
+
+
+class ForeignkeyFieldTestCase(unittest.TestCase):
+
+    class TestModel(CollectionModel):
+        collection_name = 'never_to_be_seen_again'
+
+        test_field = CharField()
+
+    def setUp(self):
+        self.database_name = 'only_foreign_key_field_123'
+        self.db = Database.create(name=self.database_name)
+
+        ForeignkeyFieldTestCase.TestModel.init()
+
+    def tearDown(self):
+        ForeignkeyFieldTestCase.TestModel.destroy()
+
+        Database.remove(name=self.database_name)
+
+    def test_basic_creation_with_default(self):
+        model = ForeignkeyFieldTestCase.TestModel()
+        field = ForeignKeyField(to=CollectionModel, default=model)
+
+        self.assertEqual(model, field.relation_model)
+
+    def test_equals(self):
+        model = ForeignkeyFieldTestCase.TestModel()
+
+        field1 = ForeignKeyField(to=CollectionModel)
+        field1.set(model)
+
+        field2 = ForeignKeyField(to=CollectionModel)
+        field2.set(model)
+
+        self.assertEqual(field1, field2)
+
+
 class TransactionTestCase(ExtendedTestCase):
     def setUp(self):
 
